/******************************************************************************

                        Recherche des nombres parfaits

                 (c) 2001/04/06 by FlashCode (Sebastien Helleu)

                           http://www.flashtux.org
                           e-mail:flashcode@flashtux.org

*******************************************************************************

==== This program is free software; you can redistribute it and/or modify
==== it under the terms of the GNU General Public License as published by
==== the Free Software Foundation; either version 2 of the License, or
==== (at your option) any later version.
====
==== This program is distributed in the hope that it will be useful,
==== but WITHOUT ANY WARRANTY; without even the implied warranty of
==== MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
==== GNU General Public License for more details.
====
==== You should have received a copy of the GNU General Public License
==== along with this program; if not, write to the Free Software
==== Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Auteur  : FlashCode <flashcode@flashtux.org>
Creation: 2001/04/06

2001/10/05: FlashCode:
    - ajout de la licence
    - reecriture du source (ajout de commentaires)

2001/10/14: Pierre Barbier de Reuille <barbier@enst.fr>
    - optimisation de l'algorihme de recherche
      (balayage de 2 a la racine carre du nombre, les autres diviseurs
      etant trouves par division)
*/


#include <stdlib.h>
#include <stdio.h>
#include <math.h>


#define MAX_DIVISEURS   10000


long    diviseurs[MAX_DIVISEURS], nb_diviseurs;


/* parfait: renvoie 1 si le nombre est parfait, 0 sinon
 *          Un nombre est parfait si la somme des ses diviseurs lui est egale
 *          Exemple: nombre parfait 6 = 1 + 2 + 3
 */

int parfait(long nombre)
{
    long    i, limite, somme_diviseurs;


    /* limite = nombre/2; */

    /* Pierre Barbier de Reuille, 2001/10/14 :
    La moiti des diviseurs sont infrieurs  la racine carre du nombre,
    les autres sont leur "complmentaire", ie. si D est un diviseur de N
    son complmentaire D1 est tel que D1 = N/D */
    limite = (int)sqrt(nombre);

    somme_diviseurs = 1;
    nb_diviseurs = 1;
    for (i = 2; i <= limite; i++)
    /*for (i = limite; i >= 2; i--)      vraiment plus rapide ??? */
        if (nombre%i == 0)
        {
            somme_diviseurs += i;

            /*diviseurs[nb_diviseurs++] = i;
            if (somme_diviseurs > nombre)
                return 0;*/

            /* Pierre Barbier de Reille, 2001/10/14 : */
            if(nombre/i != i)
                somme_diviseurs += nombre / i;
            diviseurs[nb_diviseurs++] = i;
            diviseurs[nb_diviseurs++] = nombre / i;
            if (somme_diviseurs > nombre)
                return 0;
        }
    return (somme_diviseurs == nombre);
}


int main(int argc, char *argv[])
{
    long    nbr, i;

    if ( (argc == 2) && (strcmp(argv[1], "-?") == 0) )
    {
        printf("\nPARFAIT (c) 06/10/2001 par FlashCode (Sebastien Helleu)\n");
        printf("http://flashcode.free.fr   <flashcode@free.fr>\n\n");
        printf("Syntaxe : parfait [nombre_debut]\n\n");
        exit(0);
    }
    if (argc == 2)
    {
        nbr = atoi(argv[1]);    /* debut au parametre fourni */
        if (nbr < 2)
            nbr = 2;
    }
    else
        nbr = 2;                /* sinon on commence a 2     */
    diviseurs[0] = 1;
    while (1)                   /* Ctrl-C pour sortir !      */
    {
        if (parfait(nbr))
        {
            printf("Nombre parfait: %ld (", nbr);
            for (i = 0; i < nb_diviseurs; i++)
            {
                printf("%ld", diviseurs[i]);
                if (i == nb_diviseurs-1)
                    printf(")\n");
                else
                    printf("+");
            }
        }
        if (nbr%100 == 0)
        {
            printf("Recherche... %ld\r", nbr);
            fflush(stdout);
        }
        nbr++;
    }
    return 0;
}
